<!DOCTYPE html>
<html lang="zh-cn">
<head>
    <meta charset="UTF-8">
    <title>文章新增页面</title>
    <link rel="stylesheet" href="/component/pear/css/pear.css" />
    <link rel="stylesheet" href="/admin/css/reset.css" />
    <link
  href="https://unpkg.com/@wangeditor/editor@latest/dist/css/style.css"
  rel="stylesheet"
/>
</head>
<body>

<form class="layui-form" action="">
    <input type="hidden" name="id" value="" autocomplete="off">
    <div class="mainBox">
        <div class="main-container mr-5">
            <div class="layui-form-item">
                <label class="layui-form-label required">文章标题</label>
                <div class="layui-input-block">
                    <input type="text" name="title" value="" required lay-verify="required" class="layui-input" autocomplete="off">
                </div>
            </div>
            <div class="layui-form-item">
                <label class="layui-form-label required">关键词</label>
                <div class="layui-input-block">
                    <input type="text" name="keywords" value="" required lay-verify="required" class="layui-input" autocomplete="off">
                </div>
            </div>
            <div class="layui-form-item">
                <label class="layui-form-label required">描述</label>
                <div class="layui-input-block">
                    <input type="text" name="description" value="" required lay-verify="required" class="layui-input" autocomplete="off">
                </div>
            </div>
            <div class="layui-form-item">
                <label class="layui-form-label">文章封面</label>
                <div class="layui-input-block">
                    <button type="button" class="layui-btn" id="test1">
                        <i class="layui-icon">&#xe67c;</i>上传图片
                    </button>
                    <img style="max-height: 150px;" src="" id="cover">
                    <input type="hidden" name="cover" value="" class="layui-input" autocomplete="off">
                </div>
            </div>
            <div class="layui-form-item">
                <label class="layui-form-label">文章内容</label>
                <div class="layui-input-block">
                    <div id="editor—wrapper">
                        <div id="toolbar-container"><!-- 工具栏 --></div>
                        <div id="editor-container"><!-- 编辑器 --></div>
                    </div>
                </div>
            </div>
            <div class="layui-form-item">
                <div class="button-container">
                    <button type="submit" class="pear-btn pear-btn-primary pear-btn-md" lay-submit="" lay-filter="save">
                        提交
                    </button>
                </div>
            </div>
        </div>
    </div>
</form>
<script src="/component/layui/layui.js?v=2.8.12"></script>
<script src="/component/pear/pear.js"></script>
<script src="https://unpkg.com/@wangeditor/editor@latest/dist/index.js"></script>
<script>
    // api
    const SAVE_API = "/admin/api/news/update"; // 更新新闻
    const UPLOAD_API = '/upload/image' // 上传图片
    const VIEW_API = '/admin/api/news/view'; // 查看文章信息
    const { createEditor, createToolbar } = window.wangEditor

    const editorConfig = {
        placeholder: 'Type here...',
        onChange(editor) {
        const html = editor.getHtml()
        console.log('editor content', html)
        // 也可以同步到 <textarea>
        },
        MENU_CONF: {
            uploadImage: {
                server: '/upload/image',
                fieldName: "file",
                headers: {}
            },
            editorConfig: {
                // ✅ 启用 base64 粘贴图片
                paste: {
                    // 默认关闭，要手动开启
                    allowPasteImage: true
                }
            }
        }
    }

    const editor = createEditor({
        selector: '#editor-container',
        html: '<p><br></p>',
        config: editorConfig,
        mode: 'default', // or 'simple'
    })

    const toolbarConfig = {}

    const toolbar = createToolbar({
        editor,
        selector: '#toolbar-container',
        config: toolbarConfig,
        mode: 'default', // or 'simple'
    })

    layui.use(['upload', 'jquery', 'popup'], function(){
        const upload = layui.upload;
        const $ = layui.jquery;

        initViewData()

        //执行实例
        var uploadInst = upload.render({
            elem: '#test1' //绑定元素
            ,url: UPLOAD_API // 上传接口，实际使用时改成您自己的上传接口即可。
            ,done: function(res){
            //上传完毕回调
                console.log(res)
                // 若上传失败
                if(res.code > 0){
                    return layer.msg('上传失败');
                }
                // 上传成功的一些操作
                // …
                document.querySelector('[id="cover"]').src = res.data.url
                $("input[name=cover]").val(res.data.url);
            }
            ,error: function(){
            //请求异常回调
            }
        });
    

        layui.form.on("submit(save)", function (data) {
            if (data) {
                data.field.content = editor.getHtml()
            }
            layui.$.ajax({
                url: SAVE_API,
                type: "POST",
                dateType: "json",
                data: data.field,
                success: function (res) {

                    if (res.code) {
                        return layui.popup.failure(res.msg);
                    }
                    return layui.popup.success("操作成功", function () {
                        parent.refreshTable();
                        parent.layer.close(parent.layer.getFrameIndex(window.name));
                    });
                }
            });
            return false;
        });


        function initViewData() {
            let viewId = layui.url().search['id']
            $.ajax({
                url: VIEW_API,
                data: {
                    id: viewId
                },
                method: 'get',
                dataType: "json",
                success: function (res) {

                    // 赋值表单
                    layui.each(res.data, function (key, value) {
                        let obj = $('*[name="'+key+'"]');
                        // if (key === "password") {
                        //     obj.attr("placeholder", "不更新密码请留空");
                        //     return;
                        // }
                        if (typeof obj[0] === "undefined" || !obj[0].nodeName) return;
                        if (obj[0].nodeName.toLowerCase() === "textarea") {
                            obj.html(layui.util.escape(value));
                        } else {
                            obj.attr("value", value);
                        }
                    });

                    // 文章封面
                    document.querySelector('[id="cover"]').src = res.data.cover
                    $("[name=cover]").val(res.data.cover);

                    editor.setHtml(res.data.content)

                    // ajax产生错误
                    if (res.code) {
                        layui.popup.failure(res.msg);
                    }

                }
            });
        }

    });
</script>
<style>
  #editor—wrapper {
    border: 1px solid #ccc;
    z-index: 100; /* 按需定义 */
  }
  #toolbar-container {
    border-bottom: 1px solid #ccc;
  }
  #editor-container {
    height: 500px;
  }
</style>
<style>
    .res-options {
        padding: 0 10px;
        line-height: 36px;
        white-space: nowrap;
        overflow: hidden;
        text-overflow: ellipsis;
    }
    .layui-upload-list {
        margin-top: 20px;
        display: grid;
        grid-template-columns: repeat(auto-fill, 120px);
        gap: 15px;
    }

    .img-item {
        position: relative;
        border: 1px solid #e6e6e6;
        border-radius: 4px;
        cursor: move;
    }

    .img-preview {
        width: 100%;
        height: 120px;
        object-fit: contain;
    }

    .img-operate {
        position: absolute;
        top: 5px;
        right: 5px;
        opacity: 0;
        transition: .3s;
    }

    .img-item:hover .img-operate {
        opacity: 1;
    }

    #images-tpl-container .layui-icon-close,
    #images-tpl-container .layui-icon-util {
        color: #fff;
        background: #ff5722;
        border-radius: 50%;
        padding: 2px;
        cursor: pointer;
    }

    .layui-upload-list {
        /* 保持原有样式 */
        margin-top: 20px;
        display: grid;
        grid-template-columns: repeat(auto-fill, 120px);
        gap: 15px;
    }
    /* 新增显示限制 */
    .img-item:nth-child(n+31) {
        display: none;
    }
    .load-more-btn {
        grid-column: 1 / -1;
        text-align: center;
        padding: 10px 0;
    }
</style>
</body>
</html>
